@@ -15,12 +15,18 @@
 use std::collections::HashSet;
 
 use proc_macro2::Span;
+use quote::ToTokens;
 use syn::{
     parse::{Parse, ParseStream},
-    LitStr, Token,
+    LitStr, Signature, Token,
 };
 use syn::{Ident, Result as ParseResult};
 
+use crate::{
+    directives::{EXTERN_RUST_TYPE, SUBCLASS},
+    RustPath,
+};
+
 #[derive(PartialEq, Clone, Debug, Hash)]
 pub enum UnsafePolicy {
     AllFunctionsSafe,
@@ -99,13 +105,27 @@ impl Default for Allowlist {
     }
 }
 
-#[derive(Hash, Debug)]
+#[derive(Debug)]
 pub struct Subclass {
     pub superclass: String,
     pub subclass: Ident,
 }
 
-#[derive(Hash, Debug)]
+pub struct RustFun {
+    pub path: RustPath,
+    pub sig: Signature,
+}
+
+impl std::fmt::Debug for RustFun {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        f.debug_struct("RustFun")
+            .field("path", &self.path)
+            .field("sig", &self.sig.to_token_stream().to_string())
+            .finish()
+    }
+}
+
+#[derive(Debug)]
 pub struct IncludeCppConfig {
     pub inclusions: Vec<String>,
     pub unsafe_policy: UnsafePolicy,
@@ -116,8 +136,9 @@ pub struct IncludeCppConfig {
     blocklist: Vec<String>,
     exclude_utilities: bool,
     mod_name: Option<Ident>,
-    pub rust_types: Vec<Ident>,
+    pub rust_types: Vec<RustPath>,
     pub subclasses: Vec<Subclass>,
+    pub extern_rust_funs: Vec<RustFun>,
 }
 
 impl Parse for IncludeCppConfig {
@@ -171,21 +192,15 @@ impl Parse for IncludeCppConfig {
                     syn::parenthesized!(args in input);
                     let generate: syn::LitStr = args.parse()?;
                     blocklist.push(generate.value());
-                } else if ident == "rust_type" {
+                } else if ident == "rust_type" || ident == EXTERN_RUST_TYPE {
                     let args;
                     syn::parenthesized!(args in input);
-                    let ident: syn::Ident = args.parse()?;
-                    rust_types.push(ident);
-                } else if ident == "subclass" {
+                    let id: Ident = args.parse()?;
+                    rust_types.push(RustPath::new_from_ident(id));
+                } else if ident == SUBCLASS {
                     let args;
                     syn::parenthesized!(args in input);
                     let superclass: syn::LitStr = args.parse()?;
-                    if superclass.value().contains("::") {
-                        return Err(syn::Error::new(
-                            superclass.span(),
-                            "at this time, superclasses can't be in namespaces - see https://github.com/google/autocxx/issues/599",
-                        ));
-                    }
                     args.parse::<syn::token::Comma>()?;
                     let subclass: syn::Ident = args.parse()?;
                     subclasses.push(Subclass {
@@ -237,6 +252,7 @@ impl Parse for IncludeCppConfig {
             exclude_utilities,
             mod_name,
             subclasses,
+            extern_rust_funs: Vec::new(),
         })
     }
 }
@@ -298,7 +314,7 @@ impl IncludeCppConfig {
                             .map(|sc| {
                                 [
                                     format!("{}Cpp", sc.subclass),
-                                    sc.subclass.to_string(),
+                                    sc.subclass.to_string(), // TODO may not be necessary
                                     sc.superclass.clone(),
                                 ]
                             })
@@ -331,6 +347,9 @@ impl IncludeCppConfig {
             Some(mut items) => {
                 items.any(|item| item == cpp_name)
                     || self.active_utilities().iter().any(|item| *item == cpp_name)
+                    || self.is_subclass_holder(cpp_name)
+                    || self.is_subclass_cpp(cpp_name)
+                    || self.is_rust_fun(cpp_name)
             }
         }
     }
@@ -354,7 +373,17 @@ impl IncludeCppConfig {
     }
 
     pub fn is_rust_type(&self, id: &Ident) -> bool {
-        self.rust_types.contains(id)
+        self.rust_types
+            .iter()
+            .any(|rt| rt.get_final_ident() == &id.to_string())
+            || self.is_subclass_holder(&id.to_string())
+    }
+
+    fn is_rust_fun(&self, possible_fun: &str) -> bool {
+        self.extern_rust_funs
+            .iter()
+            .map(|fun| &fun.sig.ident)
+            .any(|id| id == possible_fun)
     }
 
     pub fn superclasses(&self) -> impl Iterator<Item = &String> {
@@ -369,6 +398,12 @@ impl IncludeCppConfig {
             .any(|sc| format!("{}Holder", sc.subclass.to_string()) == id)
     }
 
+    fn is_subclass_cpp(&self, id: &str) -> bool {
+        self.subclasses
+            .iter()
+            .any(|sc| format!("{}Cpp", sc.subclass.to_string()) == id)
+    }
+
     /// Return the filename to which generated .rs should be written.
     pub fn get_rs_filename(&self) -> String {
         format!(
@@ -380,12 +415,17 @@ impl IncludeCppConfig {
         )
     }
 
-    pub fn confirm_complete(&self) -> ParseResult<()> {
+    pub fn confirm_complete(&mut self, auto_allowlist: bool) -> ParseResult<()> {
         if matches!(self.allowlist, Allowlist::Unspecified(_)) {
-            Err(syn::Error::new(
-                Span::call_site(),
-                "expected either generate! or generate_all!",
-            ))
+            if auto_allowlist {
+                self.allowlist = Allowlist::Specific(Vec::new());
+                Ok(())
+            } else {
+                Err(syn::Error::new(
+                    Span::call_site(),
+                    "expected either generate! or generate_all!",
+                ))
+            }
         } else {
             Ok(())
         }
