@@ -14,10 +14,12 @@
 
 mod config;
 pub mod file_locations;
+mod path;
 mod subclass_attrs;
 
-pub use config::{IncludeCppConfig, Subclass, UnsafePolicy};
+pub use config::{IncludeCppConfig, RustFun, Subclass, UnsafePolicy};
 use file_locations::FileLocationStrategy;
+pub use path::RustPath;
 use proc_macro2::TokenStream as TokenStream2;
 pub use subclass_attrs::SubclassAttrs;
 use syn::Result as ParseResult;
@@ -26,6 +28,15 @@ use syn::{
     Macro,
 };
 
+#[doc(hidden)]
+/// Ensure consistency between the `include_cpp!` parser
+/// and the standalone macro discoverer
+pub mod directives {
+    pub static EXTERN_RUST_TYPE: &str = "extern_rust_type";
+    pub static EXTERN_RUST_FUN: &str = "extern_rust_fun";
+    pub static SUBCLASS: &str = "subclass";
+}
+
 /// Core of the autocxx engine. See `generate` for most details
 /// on how this works.
 pub struct IncludeCpp {
